/*-------------------------------------------------------------------------
 *
 * pg_dist_node_metadata.h
 *	  definition of the relation that holds the metadata table (pg_dist_node_metadata).
 *
 * Copyright (c) Citus Data, Inc.
 *
 *-------------------------------------------------------------------------
 */

#ifndef PG_DIST_NODE_METADATA_H
#define PG_DIST_NODE_METADATA_H


/* ----------------
 *      compiler constants for pg_dist_node_metadata
 * ----------------
 */
#define Natts_pg_dist_node_metadata 1
#define Anum_pg_dist_node_metadata_metadata 1

extern int FindCoordinatorNodeId(void);

#endif /* PG_DIST_NODE_METADATA_H */
